@@ -36,11 +36,25 @@
 
 KNetworkManager* DeviceStoreDBus::_ctx = NULL;
 
-typedef struct {
-	Device * dev;
-	const char * active_net_path;
-	const char * signal_name;
-} UpdateCallbackData;
+struct UpdateCallbackData {
+    UpdateCallbackData(Device* _dev,
+                       const char* _active_net_path,
+                       const char* _signal_name)
+    {
+        dev = _dev;
+        active_net_path = qstrdup(_active_net_path);
+        signal_name = qstrdup(_signal_name);
+    }
+    ~UpdateCallbackData()
+    {
+        delete [] active_net_path;
+        delete [] signal_name;
+    }
+
+    Device * dev;
+    char * active_net_path;
+    char * signal_name;
+};
 
 void
 DeviceStoreDBus::activateDialUp (DialUp* dialup)
@@ -307,12 +321,13 @@ DeviceStoreDBus::updateNetworkCallback (DBusPendingCall* pcall, void* data)
 		//TODO: Aare these props really UTF-8? I thought they are Latin-1. It could save a pair of CPU cycles for us ;-)
 		net = sync.synchronize(QString::fromUtf8(essid), QString::fromUtf8(obj_path));
 
-		if (cbData->active_net_path)
+		if (cbData->active_net_path) {
 			if (strcmp (obj_path, cbData->active_net_path) == 0) {
 				net->setActive (true);
 			} else {
 				net->setActive (false);
 			}
+		}
 
 		//net->setObjectPath (obj_path);
 		net->setEssid (QString::fromUtf8(essid));
@@ -360,11 +375,7 @@ DeviceStoreDBus::updateNetwork (const char* obj_path, const char* net_path, cons
 
 	msg = dbus_message_new_method_call (NM_DBUS_SERVICE, net_path, NM_DBUS_INTERFACE_DEVICES, "getProperties");
 	if (msg) {
-		cbData = new UpdateCallbackData; 
-		cbData->dev = dev;
-		cbData->active_net_path = active_net_path;
-		cbData->signal_name = signal_name;
-
+		cbData = new UpdateCallbackData(dev, active_net_path, signal_name); 
 		dbus_connection_send_with_reply (con, msg, &pcall, -1);
 		if (pcall) {
 			dbus_pending_call_set_notify (pcall, DeviceStoreDBus::updateNetworkCallback, cbData, NULL);
